/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable
 * law or agreed to in writing, software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License
 * for the specific language governing permissions and limitations under the License.
 */
package javax.portlet.faces;

import javax.faces.context.FacesContext;
import javax.portlet.Event;
import javax.portlet.faces.event.EventNavigationResult;


/**
 * The <code>BridgeWriteBehindResponse</code> interface defines the api the bridge relies
 * on to acquire the buffered JSP output from the response(Wrapper) used to handle the Faces
 * implementation dependent writeBehindResponse methodlogy/interface.<p>  
 * 
 * Note: the Portlet 1.0 Bridge relied on Portlet 1.0 which didn't support response wrappers.
 * In that version the writeBehindResponse behavior is provided in a Servlet ResponseWrapper inserted
 * in a Servlet filter set up to be called on included JSPs.  In Portlet 2.0 Bridge this behavior can
 * now be implemented directly in a Portlet ResponseWrapper which can then be registered for use with the bridge.
 * So that the bridge recognizes and use this support, such wrappers must implement this interface.<p>
 * 
 * Implementations must be one of the Portlet 2.0 <code>ResponseWrappers</code> and have a null constructor that utilizes
 * <code>FacesContext.getCurrentInstance().getExternalContext().getResponse()</code>
 * to acquire the response to be wrapped.
 */

public interface BridgeWriteBehindResponse
{
  /**
   * Called by the bridge after dispatching is complete to determine whether the 
   * JSP AfterViewContent was written as chars (written via a <code>PrintWriter</code>
   * 
   * 
   * @return <code>true</code> if the response (buffer) is represented as chars
   * written via the <code>PrintWriter</code>, false otherwise.
   */
  public boolean isChars();
  
  /**
   * Called by the bridge after dispatching is complete to acquire the AfterJSPContent
   * when the response has been written as characters.  The bridge writes this buffer
   * to the (real) response.
   * 
   * @return the response as a char[].
   */
  public char[] getChars();
  
  /**
   * Called by the bridge after dispatching is complete to determine whether the 
   * JSP AfterViewContent was written as bytes (written via an <code>OutputStream</code>
   * 
   * 
   * @return <code>true</code> if the response (buffer) is represented as bytes
   * written via the <code>OutputStream</code>, false otherwise.
   */
  public boolean isBytes();
  
  /**
   * Called by the bridge after dispatching is complete to acquire the AfterJSPContent
   * when the response has been written as bytes.  The bridge writes this buffer
   * to the (real) response.
   * 
   * @return the response as a byte[].
   */ 
  public byte[] getBytes();
}
